<?php

use Tygh\Http;
use Tygh\Registry;
use Tygh\Storage;

if (!defined('BOOTSTRAP')) {
	die('Access denied');
}

if (defined('PAYMENT_NOTIFICATION')) {
	if ($mode == 'notify') {
		$MNT_COMMAND = $_REQUEST['MNT_COMMAND'];
		$order_id = explode('-', $_REQUEST['MNT_TRANSACTION_ID']);
		if (isset($order_id[0])) {
			$order_id = $order_id[0];
		}

		$order_info = fn_get_order_info($order_id);
		$processor_data = $order_info['payment_method'];
		$integrityCode = $processor_data['processor_params']['mnt_dataintegrity_code'];

		if ($MNT_COMMAND == 'CHECK') {

			if (checkSignatureRequest($_REQUEST, $integrityCode)) {
				$MNT_RESULT_CODE = 402;
				$MNT_TRANSACTION_ID = $_REQUEST['MNT_TRANSACTION_ID'];
				$MNT_ID = $processor_data['processor_params']['mnt_id'];
				$MNT_INTEGRITY_CODE = $integrityCode;
				$MNT_SIGNATURE = md5($MNT_RESULT_CODE . $MNT_ID . $MNT_TRANSACTION_ID . $MNT_INTEGRITY_CODE);

				$payerEmail = $order_info['email'];

				$products = $order_info['products'];

				$deliveryName = fn_get_shipping_name(intval($order_info['shipping_ids']));

				foreach ($products as $product) {
					$inventoryPositions[] = array(
						'name' => $product['product'],
						'price' => number_format($product['price'], 2, '.', ''),
						'quantity' => $product['amount'],
						'vatTag' => $processor_data['processor_params']['vat_mode'],
					);
				}

				// Проверяем: включена ли доставка в оплату. 			
				if (!empty($order_info['shipping_ids'])) {
					$arrayDelivery = array(
						'name' => $deliveryName,
						'price' => number_format($order_info['shipping_cost'], 2, '.', ''),
						'quantity' => "1",
						'vatTag' => "none",
					);
					array_push($inventoryPositions, $arrayDelivery);
				}

				$checkResponseArray = [
					'id' => $MNT_ID,
					'transactionId' => $MNT_TRANSACTION_ID,
					'amount' => number_format($order_info['total'], 2, '.', ''),
					'signature' => $MNT_SIGNATURE,
					'resultCode' => intval($MNT_RESULT_CODE),
					'receipt' => [
						'client' => [
							'email' => $payerEmail,
						],
						'items' => $inventoryPositions,
					],
				];

				header('Content-Type: application/json; charset=utf-8');
				echo json_encode($checkResponseArray, JSON_UNESCAPED_UNICODE);
				exit;
			} else {
				die('FAIL #1');
			}
		} else {
			if (!empty($_REQUEST['MNT_TRANSACTION_ID'])) {
				$currencies = Registry::get('currencies');
				$mnt_currency = $processor_data['processor_params']['currency'];
				if (!isset($currencies[$mnt_currency])) {
					$mnt_currency = CART_PRIMARY_CURRENCY;
				}

				$mnt_amount = number_format($order_info['total'] / $currencies[$mnt_currency]['coefficient'], 2, '.', '');

				if (
					isset($_REQUEST['MNT_ID']) && isset($_REQUEST['MNT_TRANSACTION_ID']) && isset($_REQUEST['MNT_OPERATION_ID'])
					&& isset($_REQUEST['MNT_AMOUNT']) && isset($_REQUEST['MNT_CURRENCY_CODE']) && isset($_REQUEST['MNT_TEST_MODE'])
					&& isset($_REQUEST['MNT_SIGNATURE'])
				) {
					$signature = md5("{$_REQUEST['MNT_ID']}{$_REQUEST['MNT_TRANSACTION_ID']}{$_REQUEST['MNT_OPERATION_ID']}{$_REQUEST['MNT_AMOUNT']}{$_REQUEST['MNT_CURRENCY_CODE']}{$_REQUEST['MNT_TEST_MODE']}{$integrityCode}");

					if ($_REQUEST['MNT_SIGNATURE'] == $signature && $_REQUEST['MNT_AMOUNT'] == $mnt_amount) {
						fn_change_order_status($order_id, 'P', $order_info['status']);

						// данные для кассы
						$kassa_inventory = null;
						$kassa_customer = null;
						$kassa_delivery = null;

						// ответ выдать с номенклатурой в xml
						$orderData = fn_get_order_info($order_id, false, true, false, false, DESCR_SL);
						if ($orderData) {
							$inventory = array();
							$products = $orderData['products'];
							if (is_array($products) && count($products)) {
								foreach ($products as $product) {
									$inventory[] = array('name' => trim(preg_replace("/&?[a-z0-9]+;/i", "", htmlspecialchars($product['product']))), 'price' => $product['price'], 'quantity' => $product['amount'], 'vatTag' => getVatForCheckRequest($processor_data['processor_params']['vat_mode']));
								}
							}
							if (!empty($orderData['shipping_ids'])) {
								$kassa_delivery = $orderData['shipping_cost'];
							}
							$kassa_customer = (isset($orderData['email'])) ? $orderData['email'] : null;
							$kassa_inventory = json_encode($inventory);
						}

						if ($kassa_inventory && $kassa_customer) {
							$resultCode = 200;
							header("Content-type: application/xml");
							$signature = md5($resultCode . $_REQUEST['MNT_ID'] . $_REQUEST['MNT_TRANSACTION_ID'] . $integrityCode);
							$result = '<?xml version="1.0" encoding="UTF-8" ?>';
							$result .= '<MNT_RESPONSE>';
							$result .= '<MNT_ID>' . $_REQUEST['MNT_ID'] . '</MNT_ID>';
							$result .= '<MNT_TRANSACTION_ID>' . $_REQUEST['MNT_TRANSACTION_ID'] . '</MNT_TRANSACTION_ID>';
							$result .= '<MNT_RESULT_CODE>' . $resultCode . '</MNT_RESULT_CODE>';
							$result .= '<MNT_SIGNATURE>' . $signature . '</MNT_SIGNATURE>';

							if ($kassa_inventory || $kassa_customer || $kassa_delivery) {
								$result .= '<MNT_ATTRIBUTES>';
							}

							if ($kassa_inventory) {
								$result .= '<ATTRIBUTE>';
								$result .= '<KEY>INVENTORY</KEY>';
								$result .= '<VALUE>' . $kassa_inventory . '</VALUE>';
								$result .= '</ATTRIBUTE>';
							}

							if ($kassa_customer) {
								$result .= '<ATTRIBUTE>';
								$result .= '<KEY>CUSTOMER</KEY>';
								$result .= '<VALUE>' . $kassa_customer . '</VALUE>';
								$result .= '</ATTRIBUTE>';
							}

							if ($kassa_delivery) {
								$result .= '<ATTRIBUTE>';
								$result .= '<KEY>DELIVERY</KEY>';
								$result .= '<VALUE>' . $kassa_delivery . '</VALUE>';
								$result .= '</ATTRIBUTE>';
							}

							if ($kassa_inventory || $kassa_customer || $kassa_delivery) {
								$result .= '</MNT_ATTRIBUTES>';
							}

							$result .= '</MNT_RESPONSE>';

							echo $result;
							exit;
						} else {
							die('SUCCESS');
						}
					} else {
						die('FAIL #2');
					}
				} else {
					die('FAIL #3');
				}
			} else {
				die('FAIL #4');
			}
		}
	} elseif ($mode == 'success') {

		$order_id = $_REQUEST['MNT_TRANSACTION_ID'];
		$order_info = fn_get_order_info($order_id);

		$pp_response = array();

		if ($order_info['status'] == 'N') {
			$pp_response['order_status'] = 'O';
		}
		fn_clear_cart(Tygh::$app['session']['cart']);
		fn_finish_payment($order_id, $pp_response);
		fn_order_placement_routines('orders.details?order_id=' . $order_id, $order_id);
	} elseif ($mode == 'fail') {
		if (isset($_REQUEST['MNT_TRANSACTION_ID'])) {
			$order_id = $_REQUEST['MNT_TRANSACTION_ID'];
			$order_info = fn_get_order_info($order_id);

			$pp_response = array();
			$pp_response['order_status'] = 'D';

			fn_finish_payment($order_id, $pp_response);
			fn_order_placement_routines('orders.details?order_id=' . $order_id, $order_id);
		} else {
			fn_redirect('checkout.' . (Registry::get('settings.General.checkout_style') != 'multi_page' ? 'checkout' : 'summary'));
		}
	}
} else {

	if (!isset($processor_data['processor_params']['mnt_payment_server']) || !$processor_data['processor_params']['mnt_payment_server']) {
		$processor_data['processor_params']['mnt_payment_server'] = 'www.payanyway.ru';
	}

	$action = "https://{$processor_data['processor_params']['mnt_payment_server']}/assistant.htm";

	$payment_system = $processor_data['processor_params']['payment_system'];
	$post_data = array();

	$currencies = Registry::get('currencies');
	$mnt_currency = $processor_data['processor_params']['currency'];
	if (!isset($currencies[$mnt_currency]))
		$mnt_currency = CART_PRIMARY_CURRENCY;

	if (!$mnt_currency || $mnt_currency == 'RUR')
		$mnt_currency = 'RUB';
	$updatedTransactionId = $order_id . '-' . time();
	$mnt_amount = number_format($order_info['total'] / $currencies[$mnt_currency]['coefficient'], 2, '.', '');
	$mnt_signature = md5($processor_data['processor_params']['mnt_id'] . $updatedTransactionId . $mnt_amount . $mnt_currency . $processor_data['processor_params']['mnt_test_mode'] . $processor_data['processor_params']['mnt_dataintegrity_code']);

	$post_data["MNT_ID"] = $processor_data['processor_params']['mnt_id'];
	$post_data["MNT_TRANSACTION_ID"] = strval($updatedTransactionId);
	$post_data["MNT_AMOUNT"] = $mnt_amount;
	$post_data["MNT_CURRENCY_CODE"] = $mnt_currency;
	$post_data["MNT_TEST_MODE"] = $processor_data['processor_params']['mnt_test_mode'];
	$post_data["MNT_DESCRIPTION"] = 'Оплата заказа №' . $order_id;
	$post_data["MNT_SUBSCRIBER"] = $order_info['email'];
	$post_data["MNT_SIGNATURE"] = $mnt_signature;
	$post_data["MNT_SUCCESS_URL"] = fn_url("payment_notification.success&payment=payanyway", AREA, 'current');
	$post_data["MNT_FAIL_URL"] = fn_url("payment_notification.fail&payment=payanyway", AREA, 'current');

	if ($payment_system !== "payanyway") {
		$post_data["followup"] = "true";
		$post_data["javascriptEnabled"] = "true";
	}
	if (isset($processor_data['processor_params'][$payment_system]) && !empty($processor_data['processor_params'][$payment_system]['unitId'])) {
		$post_data["paymentSystem.unitId"] = $processor_data['processor_params'][$payment_system]['unitId'];
	}
	if (isset($processor_data['processor_params'][$payment_system]) && !empty($processor_data['processor_params'][$payment_system]['accountId'])) {
		$post_data["paymentSystem.accountId"] = $processor_data['processor_params'][$payment_system]['accountId'];
	}

	switch ($payment_system) {
		case 'post':
			$post_data["additionalParameters.mailofrussiaSenderIndex"] = $order_info['payment_info']['mailofrussiaSenderIndex'];
			$post_data["additionalParameters.mailofrussiaSenderAddress"] = $order_info['payment_info']['mailofrussiaSenderAddress'];
			$post_data["additionalParameters.mailofrussiaSenderName"] = $order_info['payment_info']['mailofrussiaSenderName'];
			break;
		case 'moneymail':
			$post_data["additionalParameters.buyerEmail"] = $order_info['payment_info']['buyerEmail'];
			break;
		case 'euroset':
			$post_data["additionalParameters.rapidaPhone"] = $order_info['payment_info']['rapidaPhone'];
			break;
		case 'webmoney':
			$post_data["paymentSystem.accountId"] = $order_info['payment_info']['accountId'];
			break;
		default:
			break;
	}

	fn_create_payment_form($action, $post_data, 'PayAnyWay server', true, "get");

	exit;
}


function fn_payanyway_format_price($price, $currency) {
	$currencies = Registry::get('currencies');
	if (isset($currencies[$currency]))
		return number_format($price / $currencies[$currency]['coefficient'], 2, '.', '');
	else
		return number_format($price, 2, '.', '');
}

// Проверка подписи магазина на проверочный запрос от Монеты
function checkSignatureRequest($data, $integrityСode) {
	if (isset($data['MNT_TEST_MODE']) && !$data['MNT_TEST_MODE']) {
		$data['MNT_TEST_MODE'] = '0';
	}

	if (
		isset($data['MNT_COMMAND']) && isset($data['MNT_ID']) && isset($data['MNT_TRANSACTION_ID'])
		&& isset($data['MNT_CURRENCY_CODE']) && isset($data['MNT_TEST_MODE']) && isset($data['MNT_SIGNATURE'])
	) {
		if (!isset($data['MNT_SUBSCRIBER_ID']))
			$data['MNT_SUBSCRIBER_ID'] = '';
		if (!isset($data['MNT_OPERATION_ID']))
			$data['MNT_OPERATION_ID'] = '';
		if (!isset($data['MNT_AMOUNT']))
			$data['MNT_AMOUNT'] = '';

		$signature = md5($data['MNT_COMMAND'] . $data['MNT_ID'] . $data['MNT_TRANSACTION_ID']
			. $data['MNT_OPERATION_ID'] . $data['MNT_AMOUNT'] . $data['MNT_CURRENCY_CODE']
			. $data['MNT_SUBSCRIBER_ID'] . $data['MNT_TEST_MODE'] . $integrityСode);

		if ($data['MNT_SIGNATURE'] !== $signature) {
			return false;
		}
	} else {
		return false;
	}
	return true;
}

function getVatForCheckRequest($data) {
	$vatList = array(
		'none' => '1105',
		'vat0' => '1104',
		'vat10' => '1103',
		'vat110' => '1107',
		'vat20' => '7000',
		'vat120' => '7001',
		'vat5' => '1108',
		'vat15' => '1110',
		'vat7' => '1109',
		'vat17' => '1111',
	);

	return isset($vatList[$data]) ? $vatList[$data] : '1105';
}
